import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.images.ColorImage;
import imagescience.images.Dimensions;
import imagescience.images.Image;
import imagescience.transforms.Shift;
import imagescience.utilities.I5DResource;
import imagescience.utilities.Messenger;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.image.ColorModel;

public class TJ_Shift implements PlugIn {

    private static String zshift = "0.0";
    private static String yshift = "0.0";
    private static String xshift = "0.0";

    private static final String[] schemes = {
	"nearest neighbor",
	"linear",
	"cubic convolution",
	"cubic B-spline",
	"cubic O-MOMS",
	"quintic B-spline"
    };
    private static int scheme = 1;

    private static String bgvalue = "0.0";

    public void run(String arg) {

	msg.debug(TJ_Options.debug);
	msg.debug("TransformJ "+TJ_Options.VERSION+": Shift");

	if (IJ.versionLessThan("1.35i")) return;

	final ImagePlus imp = WindowManager.getCurrentImage();
	if (imp == null)  { error("There are no images open"); return; }

	GenericDialog gd = new GenericDialog("TransformJ: Shift");
	gd.addStringField("x-shift in pixels:",xshift);
	gd.addStringField("y-shift in pixels:",yshift);
	gd.addStringField("z-shift in slices:",zshift);
	gd.addPanel(new Panel(),GridBagConstraints.WEST,new Insets(0,0,0,0));
	gd.addChoice("Interpolation scheme:",schemes,schemes[scheme]);
	gd.addStringField("Background value:",bgvalue);
	gd.showDialog();

	if (gd.wasCanceled()) return;

	xshift = gd.getNextString();
	yshift = gd.getNextString();
	zshift = gd.getNextString();
	scheme = gd.getNextChoiceIndex();
	bgvalue = gd.getNextString();

	try {
	    final Image img = Image.wrap(imp);
	    final Shift shifter = new Shift();
	    shifter.debug(TJ_Options.debug);
	    double xs, ys, zs, bg;
	    try { xs = Double.parseDouble(xshift); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid x-shift value"); }
	    try { ys = Double.parseDouble(yshift); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid y-shift value"); }
	    try { zs = Double.parseDouble(zshift); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid z-shift value"); }
	    try { bg = Double.parseDouble(bgvalue); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid background value"); }
	    shifter.background(bg);
	    int ischeme = Shift.NEAREST;
	    switch (scheme) {
	    case 0: ischeme = Shift.NEAREST; break;
	    case 1: ischeme = Shift.LINEAR; break;
	    case 2: ischeme = Shift.CUBIC; break;
	    case 3: ischeme = Shift.BSPLINE3; break;
	    case 4: ischeme = Shift.OMOMS3; break;
	    case 5: ischeme = Shift.BSPLINE5; break;
	    }
	    final Image newimg = shifter.run(img,xs,ys,zs,ischeme);
	    ImagePlus newimp = newimg.imageplus();
	    newimp.setCalibration(imp.getCalibration().copy());
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    newip.setMinAndMax(ip.getMin(),ip.getMax());
	    boolean i5dinstalled = false;
	    try { Class.forName("Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		final Dimensions newdims = newimg.dimensions();
		if (newdims.t > 1 || newdims.s > 1) {
		    newimp = I5DResource.convert(newimp,true);
		    if (I5DResource.instance(imp))
			I5DResource.transfer(imp,newimp);
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    error("Not enough memory for this operation");

	} catch (IllegalArgumentException e) {
	    error(e.getMessage());
	}
    }

    private void error(final String message) { msg.error("TransformJ: Error", message+"."); }

    private final Messenger msg = new Messenger();

}
